'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os

import pygame
from pygame.transform import rotate, flip


data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

def load_background():
    return pygame.image.load(filepath("background.png")).convert()

def load_ant_surfaces():
    surface = pygame.image.load(filepath("ant.png")).convert()
    surface.set_colorkey(surface.get_at((0, 0)))
    w, h = surface.get_width() // 4, surface.get_height()
    surfaces_0 = [surface.subsurface(pygame.Rect((i*w, 0), (w, h)))
                  for i in xrange(2)]
    surfaces_60 = [surface.subsurface(pygame.Rect((i*w, 0), (w, h)))
                  for i in xrange(2, 4)]
    surfaces = [surfaces_0,
                [flip(rotate(s, 90), 1, 0) for s in surfaces_60],
                surfaces_60,
                [rotate(s, 90) for s in surfaces_0],
                [flip(s, 1, 0) for s in surfaces_60],
                [rotate(s, 90) for s in surfaces_60],
                [rotate(s, 180) for s in surfaces_0],
                [flip(rotate(s, 90), 0, 1) for s in surfaces_60],
                [rotate(s, 180) for s in surfaces_60],
                [rotate(s, 270) for s in surfaces_0],
                [flip(s, 0, 1) for s in surfaces_60],
                [rotate(s, 270) for s in surfaces_60],
               ]
    return surfaces

_sounds = None

def load_sounds():
    global _sounds
    if pygame.mixer.get_init():
        snap = pygame.mixer.Sound(filepath("snap.wav"))
        splash = pygame.mixer.Sound(filepath("splash.wav"))
        _sounds = {"snap": snap,
                   "splash": splash}

def get_sound(name):
    global _sounds
    if _sounds:
        return _sounds[name]
