'''Game main module.

Contains the entry point used by the run_game.py script.

Feel free to put all your game code here, or in other modules in this "gamelib"
package.
'''

from itertools import chain, izip
from operator import itemgetter
import sys

import pygame
pygame.init()

import data
from .objects import Peg, String, Link, StringLink, Ant, AntFactory
from .paths import SimplePathfinder
import ui

def main():
    if not pygame.image.get_extended():
        print "This game requires PyGame to be built with extended image support."
        sys.exit()

    size = width, height = 800, 600
    screen = pygame.display.set_mode(size)
    pygame.display.set_caption("String Game")
    #pygame.display.set_icon(icon)

    playing_surface = screen.subsurface(pygame.Rect((0, 0), (width, height-100)))
    toolbar_surface = screen.subsurface(pygame.Rect((0, height-100), (width, 100)))

    playing_area = ui.PlayingArea(playing_surface)
    state = GameState(ui.Toolbar(toolbar_surface))
    mouse_controller = ui.MouseController(state)
    handlers = {pygame.QUIT: evt_quit,
                pygame.ACTIVEEVENT: mouse_controller.evt_active,
                pygame.MOUSEBUTTONUP: mouse_controller.evt_mousebuttonup,
                pygame.MOUSEBUTTONDOWN: mouse_controller.evt_mousebuttondown}

    pygame.event.set_allowed(handlers.keys())

    ant_surfaces = data.load_ant_surfaces()

    if pygame.mixer.get_init():
        data.load_sounds()

    clock = pygame.time.Clock()

    pegs = state.pegs
    pegs.append(Peg((-20, 100)))
    pegs.append(Peg((100, 100)))

    pegs.append(Peg((700, 100)))
    pegs.append(Peg((820, 100)))

    pegs.append(Peg((-20, 400)))
    pegs.append(Peg((100, 400)))

    pegs.append(Peg((700, 400)))
    pegs.append(Peg((820, 400)))

    links = state.links
    links.append(Link(pegs[0], pegs[1]))
    links.append(Link(pegs[2], pegs[3]))
    links.append(Link(pegs[4], pegs[5]))
    links.append(Link(pegs[6], pegs[7]))

    ants = state.ants
    ant_factories = state.ant_factories
    ant_factories.append(AntFactory(links[0], 0, pegs[1], 1, 1, 1,
                                    SimplePathfinder, ant_surfaces, 80))
    ant_factories.append(AntFactory(links[2], 0, pegs[5], 1, 1, 1,
                                    SimplePathfinder, ant_surfaces, 80))

    links[1].is_goal = True

    state.dirty = [screen.get_rect()]

    ttf = target_fps = 40

    while True:
        for event in pygame.event.get():
            if event.type in handlers:
                handlers[event.type](event)

        for object in chain(ants, links, ant_factories):
            if object.is_alive:
                object.act(state)
            if not object.is_alive:
                state.dirty.append(object.rect)
                object.remove(state)

        playing_area.draw()
        state.toolbar.draw()
        for object in chain(links, pegs, ants):
            object.draw(playing_area)
        mouse_controller.do_drawing(playing_area)
        pygame.display.update(state.dirty)
        state.dirty = []

        clock.tick(target_fps)
        ttf -= 1
        if ttf <= 0:
            ttf = target_fps
            print clock.get_fps(), clock.get_rawtime()


def evt_quit(event):
    sys.exit()


class GameState(object):

    def __init__(self, toolbar):
        self.dirty = []
        self.pegs = []
        self.links = []
        self.ants = []
        self.ant_factories = []
        self.toolbar = toolbar
