"Path-finding logic for Ants."

from collections import deque


class Pathfinder(object):

    "Interface for ant pathfinding."

    def __init__(self):
        self.path = None

    def find_path(self, source, next_peg):
        pass

    def get_path(self, source, next_peg):
        if self.path_needed(source):
            self.path = self.find_path(source, next_peg)
        assert self.path, "Pathfinding failed"
        if source is self.path[-1][1]:
            self.path.pop()
        return self.path

    def follow_path(self, source, next_peg):
        return self.get_path(source, next_peg)[-1][1]

    def path_needed(self, source):
        if not self.path:
            return True
        if not all(link and link.is_alive for (peg, link) in self.path):
            return True
        return False

    def distance(self, peg):
        return peg.x


class SimplePathfinder(Pathfinder):

    "Simple breadth-first search pathfinder."

    def find_path(self, source, next_peg):
        if source.is_goal:
            return [(next_peg, None)]
        closest_distance = self.distance(next_peg)
        closest_path = [(next_peg, None)]
        open = deque()
        for next_link in next_peg.links:
            if next_link is not source:
                open.append([(next_peg, next_link)])
        closed = set([(source, next_peg)])

        while open:
            path = open.popleft()
            peg, link = path[-1]
            next_peg = link.other_peg(peg)
            if (link, next_peg) in closed:
                continue
            if link.is_goal:
                path.append((next_peg, None))
                path.reverse()
                return path
            distance = self.distance(next_peg)
            if distance > closest_distance:
                closest_distance = distance
                closest_path = path + [(next_peg, None)]
            for next_link in next_peg.links:
                if next_link is not link:
                    open.append(path + [(next_peg, next_link)])
            closed.add((link, next_peg))

        closest_path.reverse()
        return closest_path
